@@ -16,112 +16,96 @@
 
 /* ----------------------------------------------------------------------- */
 
-#if 0
-static char *content_group[16] = {
-    [ 0x1 ] = "Movie/Drama",
-    [ 0x2 ] = "News/Current affairs",
-    [ 0x3 ] = "Show/Game show",
-    [ 0x4 ] = "Sports",
-    [ 0x5 ] = "Children's/Youth programmes",
-    [ 0x6 ] = "Music/Ballet/Dance",
-    [ 0x7 ] = "Arts/Culture (without music)",
-    [ 0x8 ] = "Social/Political issues/Economics",
-    [ 0x9 ] = "Education/ Science/Factual topics",
-    [ 0xA ] = "Leisure hobbies",
-    [ 0xB ] = "Special Characteristics",
-};
-#endif
-
 static char *content_desc[256] = {
-    [ 0x10 ] = "movie/drama (general)",
-    [ 0x11 ] = "detective/thriller",
-    [ 0x12 ] = "adventure/western/war",
-    [ 0x13 ] = "science fiction/fantasy/horror",
-    [ 0x14 ] = "comedy",
-    [ 0x15 ] = "soap/melodrama/folkloric",
-    [ 0x16 ] = "romance",
-    [ 0x17 ] = "serious/classical/religious/historical movie/drama",
-    [ 0x18 ] = "adult movie/drama",
+    [ 0x10 ] = "Film|movie/drama (general)",
+    [ 0x11 ] = "Film|detective/thriller",
+    [ 0x12 ] = "Film|adventure/western/war",
+    [ 0x13 ] = "Film|science fiction/fantasy/horror",
+    [ 0x14 ] = "Film|comedy",
+    [ 0x15 ] = "Film|soap/melodrama/folkloric",
+    [ 0x16 ] = "Film|romance",
+    [ 0x17 ] = "Film|serious/classical/religious/historical movie/drama",
+    [ 0x18 ] = "Film|adult movie/drama",
 
-    [ 0x20 ] = "news/current affairs (general)",
-    [ 0x21 ] = "news/weather report",
-    [ 0x22 ] = "news magazine",
-    [ 0x23 ] = "documentary",
-    [ 0x24 ] = "discussion/interview/debate",
+    [ 0x20 ] = "News|news/current affairs (general)",
+    [ 0x21 ] = "News|news/weather report",
+    [ 0x22 ] = "News|news magazine",
+    [ 0x23 ] = "News|documentary",
+    [ 0x24 ] = "News|discussion/interview/debate",
 
-    [ 0x30 ] = "show/game show (general)",
-    [ 0x31 ] = "game show/quiz/contest",
-    [ 0x32 ] = "variety show",
-    [ 0x33 ] = "talk show",
+    [ 0x30 ] = "Show|show/game show (general)",
+    [ 0x31 ] = "Show|game show/quiz/contest",
+    [ 0x32 ] = "Show|variety show",
+    [ 0x33 ] = "Show|talk show",
 
-    [ 0x40 ] = "sports (general)",
-    [ 0x41 ] = "special events (Olympic Games, World Cup etc.)",
-    [ 0x42 ] = "sports magazines",
-    [ 0x43 ] = "football/soccer",
-    [ 0x44 ] = "tennis/squash",
-    [ 0x45 ] = "team sports (excluding football)",
-    [ 0x46 ] = "athletics",
-    [ 0x47 ] = "motor sport",
-    [ 0x48 ] = "water sport",
-    [ 0x49 ] = "winter sports",
-    [ 0x4A ] = "equestrian",
-    [ 0x4B ] = "martial sports",
+    [ 0x40 ] = "Sports|sports (general)",
+    [ 0x41 ] = "Sports|special events (Olympic Games, World Cup etc.)",
+    [ 0x42 ] = "Sports|sports magazines",
+    [ 0x43 ] = "Sports|football/soccer",
+    [ 0x44 ] = "Sports|tennis/squash",
+    [ 0x45 ] = "Sports|team sports (excluding football)",
+    [ 0x46 ] = "Sports|athletics",
+    [ 0x47 ] = "Sports|motor sport",
+    [ 0x48 ] = "Sports|water sport",
+    [ 0x49 ] = "Sports|winter sports",
+    [ 0x4A ] = "Sports|equestrian",
+    [ 0x4B ] = "Sports|martial sports",
 
-    [ 0x50 ] = "children's/youth programmes (general)",
-    [ 0x51 ] = "pre-school children's programmes",
-    [ 0x52 ] = "entertainment programmes for 6 to 14",
-    [ 0x53 ] = "entertainment programmes for 10 to 16",
-    [ 0x54 ] = "informational/educational/school programmes",
-    [ 0x55 ] = "cartoons/puppets",
+    [ 0x50 ] = "Children|children's/youth programmes (general)",
+    [ 0x51 ] = "Children|pre-school children's programmes",
+    [ 0x52 ] = "Children|entertainment programmes for 6 to 14",
+    [ 0x53 ] = "Children|entertainment programmes for 10 to 16",
+    [ 0x54 ] = "Children|informational/educational/school programmes",
+    [ 0x55 ] = "Children|cartoons/puppets",
 
-    [ 0x60 ] = "music/ballet/dance (general)",
-    [ 0x61 ] = "rock/pop",
-    [ 0x62 ] = "serious music/classical music",
-    [ 0x63 ] = "folk/traditional music",
-    [ 0x64 ] = "jazz",
-    [ 0x65 ] = "musical/opera",
-    [ 0x66 ] = "ballet",
+    [ 0x60 ] = "Music|music/ballet/dance (general)",
+    [ 0x61 ] = "Music|rock/pop",
+    [ 0x62 ] = "Music|serious music/classical music",
+    [ 0x63 ] = "Music|folk/traditional music",
+    [ 0x64 ] = "Music|jazz",
+    [ 0x65 ] = "Music|musical/opera",
+    [ 0x66 ] = "Music|ballet",
 
-    [ 0x70 ] = "arts/culture (without music, general)",
-    [ 0x71 ] = "performing arts",
-    [ 0x72 ] = "fine arts",
-    [ 0x73 ] = "religion",
-    [ 0x74 ] = "popular culture/traditional arts",
-    [ 0x75 ] = "literature",
-    [ 0x76 ] = "film/cinema",
-    [ 0x77 ] = "experimental film/video",
-    [ 0x78 ] = "broadcasting/press",
-    [ 0x79 ] = "new media",
-    [ 0x7A ] = "arts/culture magazines",
-    [ 0x7B ] = "fashion",
+    [ 0x70 ] = "Arts|arts/culture (without music, general)",
+    [ 0x71 ] = "Arts|performing arts",
+    [ 0x72 ] = "Arts|fine arts",
+    [ 0x73 ] = "Arts|religion",
+    [ 0x74 ] = "Arts|popular culture/traditional arts",
+    [ 0x75 ] = "Arts|literature",
+    [ 0x76 ] = "Arts|film/cinema",
+    [ 0x77 ] = "Arts|experimental film/video",
+    [ 0x78 ] = "Arts|broadcasting/press",
+    [ 0x79 ] = "Arts|new media",
+    [ 0x7A ] = "Arts|arts/culture magazines",
+    [ 0x7B ] = "Arts|fashion",
 
-    [ 0x80 ] = "social/political issues/economics (general)",
-    [ 0x81 ] = "magazines/reports/documentary",
-    [ 0x82 ] = "economics/social advisory",
-    [ 0x83 ] = "remarkable people",
+    [ 0x80 ] = "Social|social/political issues/economics (general)",
+    [ 0x81 ] = "Social|magazines/reports/documentary",
+    [ 0x82 ] = "Social|economics/social advisory",
+    [ 0x83 ] = "Social|remarkable people",
 
-    [ 0x90 ] = "education/science/factual topics (general)",
-    [ 0x91 ] = "nature/animals/environment",
-    [ 0x92 ] = "technology/natural sciences",
-    [ 0x93 ] = "medicine/physiology/psychology",
-    [ 0x94 ] = "foreign countries/expeditions",
-    [ 0x95 ] = "social/spiritual sciences",
-    [ 0x96 ] = "further education",
-    [ 0x97 ] = "languages",
+    [ 0x90 ] = "Education|education/science/factual topics (general)",
+    [ 0x91 ] = "Education|nature/animals/environment",
+    [ 0x92 ] = "Education|technology/natural sciences",
+    [ 0x93 ] = "Education|medicine/physiology/psychology",
+    [ 0x94 ] = "Education|foreign countries/expeditions",
+    [ 0x95 ] = "Education|social/spiritual sciences",
+    [ 0x96 ] = "Education|further education",
+    [ 0x97 ] = "Education|languages",
 
-    [ 0xA0 ] = "leisure hobbies (general)",
-    [ 0xA1 ] = "tourism/travel",
-    [ 0xA2 ] = "handicraft",
-    [ 0xA3 ] = "motoring",
-    [ 0xA4 ] = "fitness & health",
-    [ 0xA5 ] = "cooking",
-    [ 0xA6 ] = "advertizement/shopping",
-    [ 0xA7 ] = "gardening",
+    [ 0xA0 ] = "Leisure|leisure hobbies (general)",
+    [ 0xA1 ] = "Leisure|tourism/travel",
+    [ 0xA2 ] = "Leisure|handicraft",
+    [ 0xA3 ] = "Leisure|motoring",
+    [ 0xA4 ] = "Leisure|fitness & health",
+    [ 0xA5 ] = "Leisure|cooking",
+    [ 0xA6 ] = "Leisure|advertizement/shopping",
+    [ 0xA7 ] = "Leisure|gardening",
 
-    [ 0xB0 ] = "original language",
-    [ 0xB1 ] = "black & white",
-    [ 0xB2 ] = "unpublished",
-    [ 0xB3 ] = "live broadcast",
+    [ 0xB0 ] = "Special|original language",
+    [ 0xB1 ] = "Special|black & white",
+    [ 0xB2 ] = "Special|unpublished",
+    [ 0xB3 ] = "Special|live broadcast",
 };
 
 /* ----------------------------------------------------------------------- */
